using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Tao.OpenGl;
using InfoVis.Tree;

namespace InfoVis
{
    /**
     * The Window Class. It Handles all the Relevant user Interaction
     */
    public partial class InfoVis : Form
    {
        private int location;
        private GiWEngine.Graphics.OpenGLCamera mCamera;
        private int mCurrentX, mLastX, mCurrentY, mLastY;
        private Boolean mMoveMouse;
        private Tree.Tree mTree;
        private Boolean radialTree;
        private TreeLayout mTreeLayout;
        private Boolean mLayoutLoaded;
        private Boolean mVaryingAlpha;
        private GiWEngine.Base.Logger mLogger;
        private GiWEngine.Graphics.FrameBufferObject mFbo1;
        private GiWEngine.Graphics.FrameBufferObject mFbo2;
        //private GiWEngine.Graphics.HDRFrameBufferObject fullSizeFBO;
        private GiWEngine.Graphics.Effect mEffect;
        private string lastresult;
        private Tree.TreeNode mSelectedNode;
        private Dictionary<string, int> mNodes;
        private Boolean selectionMode;
        private GiWEngine.Math.Vector3 StartPoint;
        private GiWEngine.Math.Vector3 EndPoint;
        private GiWEngine.Math.Vector3 HoverPoint;

        /**
         * CTR of Form. Creates the OpenGl Context too
         */
        public InfoVis()
        {
            InitializeComponent();
            this.simpleOpenGlControl1.InitializeContexts();
            initGl();
            //this.KeyUp += new KeyEventHandler(this.Form_KeyUp);                 // On KeyUp Event Call Form_KeyUp
            this.simpleOpenGlControl1.MouseMove += new MouseEventHandler(this.infoVisMouseMove);        // On MouseMove Event Cal giwMouseMove
            this.simpleOpenGlControl1.MouseDown += new MouseEventHandler(this.infoVisMouseDown);
            this.simpleOpenGlControl1.MouseWheel += new MouseEventHandler(this.infoVisMouseWheel);
            location = 0;
            mMoveMouse = false;
            radialTree = true;
            mLayoutLoaded = false;
            radioButton1.Checked = radialTree;
            mVaryingAlpha = checkBox1.Checked;
            //new GiWEngine.Base.EngineCore();
            mLogger = GiWEngine.Base.EngineCore.Instance.logger;
            GiWEngine.Base.EngineCore.Instance.loadAllExtensions();
            GiWEngine.Graphics.Effect.checkCgProfiles(mLogger);
            GiWEngine.Base.EngineCore.Instance.loadAllExtensions();
            mFbo1 = new GiWEngine.Graphics.FrameBufferObject(this.simpleOpenGlControl1.Width, this.simpleOpenGlControl1.Height, 1);
            mFbo2 = new GiWEngine.Graphics.FrameBufferObject(this.simpleOpenGlControl1.Width, this.simpleOpenGlControl1.Height, 1);
            //fullSizeFBO = new GiWEngine.Graphics.HDRFrameBufferObject(this.simpleOpenGlControl1.Width, this.simpleOpenGlControl1.Height);
            mEffect = GiWEngine.Base.EngineCore.Instance.effMgrContent.getValue("shader/fbo");
            lastresult = "";
            selectionMode = checkBox3.Checked;

        }

        /**
         * Callback function for the mousedown Event
         * Handles Also picking and selection
         * @param sender Object sender
         * @param m Mouse Event
         */
        private void infoVisMouseDown(object sender, System.Windows.Forms.MouseEventArgs m)
        {
            if (m.Button == MouseButtons.Left)
            {
                //mCamera.setView();
                /*if (mCamera != null)
                {
                    mCamera.setCamera();
                }*/
                //recalc y into ogl
                int tmpY = this.simpleOpenGlControl1.Height - mCurrentY;
                //Console.WriteLine(mCurrentY);

                //Get matrices
                //They must have been set correctly!
                double[] modelMatrix = new double[16];
                double[] projMatrix = new double[16];
                int[] viewport = new int[4];

                Gl.glGetDoublev(Gl.GL_MODELVIEW_MATRIX, modelMatrix);
                Gl.glGetDoublev(Gl.GL_PROJECTION_MATRIX, projMatrix);
                Gl.glGetIntegerv(Gl.GL_VIEWPORT, viewport);

                //Handle picking

                double Xd, Yd, Zd, X0, Y0, Z0;

                //Get point under mouse cursor, depth = 0.5 (somewhere between near/far plane)
                Glu.gluUnProject(mCurrentX, tmpY, 0.5, modelMatrix, projMatrix, viewport, out Xd, out Yd, out Zd);

                //Get depth (Z0) of object-point on near-plane
                Glu.gluProject(0, 0, 0, modelMatrix, projMatrix, viewport, out X0, out Y0, out Z0);

                //Use Z0 to get point posx,posy,Zd on near plane
                Glu.gluUnProject(mCurrentX, tmpY, Z0, modelMatrix, projMatrix, viewport, out X0, out Y0, out Z0);

                //Create vectors
                //p0: Point on near-plane
                GiWEngine.Math.Vector3 p0 = new GiWEngine.Math.Vector3((float)X0, (float)Y0, (float)Z0);
                //Pd: Point somewhere between near/far-plane
                GiWEngine.Math.Vector3 pd = new GiWEngine.Math.Vector3((float)Xd, (float)Yd, (float)Zd);
                
                
                if (!selectionMode)
                {
                    if (mTreeLayout != null)
                    {
                        //select node
                        mSelectedNode = mTreeLayout.select(p0, pd);
                        if (mTreeLayout != null)
                        {
                            mTreeLayout.setSelectedNode(mSelectedNode);
                        }
                    }
                }
                else if (selectionMode && (StartPoint == null))
                {
                    GiWEngine.Math.Vector3 ray = pd - p0;
                    float t = -p0.z / ray.z;
                    StartPoint = new GiWEngine.Math.Vector3(p0.x + t * ray.x, p0.y + t * ray.y, 0.0f);
                    //Console.WriteLine("StartPoint: " + StartPoint);
                    mTreeLayout.selectWithLine(StartPoint, null);
                }
                else if (selectionMode && (StartPoint != null) && (EndPoint == null))
                {
                    GiWEngine.Math.Vector3 ray = pd - p0;
                    float t = -p0.z / ray.z;
                    EndPoint = new GiWEngine.Math.Vector3(p0.x + t * ray.x, p0.y + t * ray.y, 0.0f);
                    HoverPoint = null;
                    mTreeLayout.selectWithLine(StartPoint, EndPoint);
                    //Console.WriteLine("EndPoint: " + EndPoint);
                }
                else if (selectionMode && (StartPoint != null) && (EndPoint != null))
                {
                    mTreeLayout.selectWithLine(null, null);
                    StartPoint = null;
                    EndPoint = null;
                }
                /*else if (selectionMode && (EndPoint != null))
                {
                    GiWEngine.Math.Vector3 ray = pd - p0;
                    float t = -p0.z / ray.z;
                    StartPoint = new GiWEngine.Math.Vector3(p0.x + t * ray.x, p0.y + t * ray.y, 0.0f);
                    EndPoint = null;
                    mTreeLayout.selectWithLine(StartPoint, null);
                    //Console.WriteLine("StartPoint: " + StartPoint);
                }*/
            }
            /*if ((m.Button == MouseButtons.Right) && selectionMode)
            {
                mTreeLayout.selectWithLine(null, null);
                StartPoint = null;
                EndPoint = null;
            }*/
        }

        /**
         * Callback Function of MouseWheel
         * Handles the Zoom Factor
         * @param sender Object sender
         * @param m Mouse Event
         */
        private void infoVisMouseWheel(object sender, System.Windows.Forms.MouseEventArgs m)
        {
            if (m.Delta != 0)
            {
                GiWEngine.Math.Vector3 mWheel = new GiWEngine.Math.Vector3(0.0f, 0.0f, (float)(m.Delta) / 100.0f);
                mCamera.mCamPosContent += mWheel;
                //Console.WriteLine("CamPos: " + mCamera.mCamPosContent);
                if (mCamera != null)
                {
                    mCamera.setCamera();
                }
            }
        }

        /**
         * Callback Function of MouseMove
         * Handles the Panning and some part of the Picking Code
         * @param sender Object sender
         * @param m Mouse Event
         */
        private void infoVisMouseMove(object sender, System.Windows.Forms.MouseEventArgs m)
        {
            mLastX = mCurrentX;
            mLastY = mCurrentY;
            mCurrentX = m.X;
            mCurrentY = m.Y;
            //Console.WriteLine("(" + mCurrentX + "," + mCurrentY + ")");
            if (m.Button == MouseButtons.Right)
            {

                int zeroX = this.simpleOpenGlControl1.Location.X + (int)Math.Floor((((double)this.simpleOpenGlControl1.Width) / 2.0));
                int zeroY = this.simpleOpenGlControl1.Location.Y + (int)Math.Floor((((double)this.simpleOpenGlControl1.Height) / 2.0));

                int xWay = mCurrentX - mLastX;
                int yWay = mCurrentY - mLastY;

                /*Point pt = new Point(); ;
                pt.X = zeroX;
                pt.Y = zeroY;
                Point p = this.simpleOpenGlControl1.PointToScreen(pt);
                Cursor.Position = p;*/
                //if ((this.simpleOpenGlControl1.Location.X < m.X) && (m.X < this.simpleOpenGlControl1.Location.X + this.simpleOpenGlControl1.Width) &&
                //    (this.simpleOpenGlControl1.Location.Y < m.Y) && (m.Y < this.simpleOpenGlControl1.Location.Y + this.simpleOpenGlControl1.Height))
                //{

                GiWEngine.Math.Vector3 wayVec = new GiWEngine.Math.Vector3(((float)xWay) / 25.0f, ((float)yWay) / 25.0f, 0.0f);
                mCamera.mCamPosContent += wayVec;
                //Console.WriteLine("CamPos: " + mCamera.mCamPosContent);
                mCamera.mFocusPointContent += wayVec;
                //Console.WriteLine("FocusPoint: " + mCamera.mFocusPointContent);

                //}
                if (mCamera != null)
                {
                    mCamera.setCamera();
                }
            }
            else
            {
                
                //recalc y into ogl
                int tmpY = this.simpleOpenGlControl1.Height - mCurrentY;
                //Console.WriteLine(mCurrentY);

                //Get matrices
                //They must have been set correctly!
                double[] modelMatrix = new double[16];
                double[] projMatrix = new double[16];
                int[] viewport = new int[4];

                Gl.glGetDoublev(Gl.GL_MODELVIEW_MATRIX, modelMatrix);
                Gl.glGetDoublev(Gl.GL_PROJECTION_MATRIX, projMatrix);
                Gl.glGetIntegerv(Gl.GL_VIEWPORT, viewport);

                //Handle picking

                double Xd, Yd, Zd, X0, Y0, Z0;

                //Get point under mouse cursor, depth = 0.5 (somewhere between near/far plane)
                Glu.gluUnProject(mCurrentX, tmpY, 0.5, modelMatrix, projMatrix, viewport, out Xd, out Yd, out Zd);

                //Get depth (Z0) of object-point on near-plane
                Glu.gluProject(0, 0, 0, modelMatrix, projMatrix, viewport, out X0, out Y0, out Z0);

                //Use Z0 to get point posx,posy,Zd on near plane
                Glu.gluUnProject(mCurrentX, tmpY, Z0, modelMatrix, projMatrix, viewport, out X0, out Y0, out Z0);

                //Create vectors
                //p0: Point on near-plane
                GiWEngine.Math.Vector3 p0 = new GiWEngine.Math.Vector3((float)X0, (float)Y0, (float)Z0);
                //Pd: Point somewhere between near/far-plane
                GiWEngine.Math.Vector3 pd = new GiWEngine.Math.Vector3((float)Xd, (float)Yd, (float)Zd);
                //Console.WriteLine(this.toolTip1.ToString());
                /*Gl.glColor3f(1.0f, 1.0f, 0.0f);
                Gl.glBegin(Gl.GL_LINES);
                Gl.glVertex3fv(p0.toFloatArray());
                Gl.glVertex3fv(pd.toFloatArray());
                Gl.glEnd();
                */
                if (selectionMode && (EndPoint == null) && (StartPoint != null))
                {
                    GiWEngine.Math.Vector3 ray = pd - p0;
                    float t = -p0.z / ray.z;
                    HoverPoint = new GiWEngine.Math.Vector3(p0.x + t * ray.x, p0.y + t * ray.y, 0.0f);
                }
                if (mTreeLayout != null)
                {
                    lastresult = mTreeLayout.rayPick(p0, pd);
                }
            }
        }

        /*private void infoVisKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {

            if (e.KeyCode == Keys.Up || e.KeyCode == Keys.W)
            {
                mCamera.moveForward(0.1f);
            }
            else if (e.KeyCode == Keys.Down || e.KeyCode == Keys.S)
            {
                mCamera.moveForward(-0.1f);
            }
            else if (e.KeyCode == Keys.Left || e.KeyCode == Keys.A)
            {
                mCamera.moveSideWards(-0.1f);
            }
            else if (e.KeyCode == Keys.Right || e.KeyCode == Keys.D)
            {
                mCamera.moveSideWards(0.1f);
            }
            else if (e.KeyCode == Keys.O)
            {
                optimized = !optimized;
                //mSoundSystem.playSound("sounds/test1.wav", false);
            }
            else if (e.KeyCode == Keys.C)
            {
                if (mScene.shotsContent[0].cameraContent != mFlyCamera)
                {
                    mCamera = mScene.shotsContent[0].cameraContent;
                    mScene.shotsContent[0].cameraContent = mFlyCamera;
                }
                else
                {
                    mFlyCamera = mScene.shotsContent[0].cameraContent;
                    mScene.shotsContent[0].cameraContent = mCamera;
                }
            }
            else if (e.KeyCode == Keys.F3)
            {
                wireframe = !wireframe;
                if (wireframe)
                {
                    Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_LINE);
                }
                else
                {
                    Gl.glPolygonMode(Gl.GL_FRONT_AND_BACK, Gl.GL_FILL);
                }
            }
            else
            if (e.KeyCode == Keys.Escape)
            {
                this.Close();
            }
        }
        */

        /**
         * Inits the OpenGl Context
         */
        private void initGl()
        {
            mCamera = new GiWEngine.Graphics.OpenGLCamera(this.simpleOpenGlControl1.Width, this.simpleOpenGlControl1.Height, true);
            Gl.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
            Gl.glShadeModel(Gl.GL_FLAT);
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            Gl.glEnable(Gl.GL_CULL_FACE);
            //Gl.glEnable(Gl.GL_LINE_SMOOTH);
            //Gl.glHint(Gl.GL_LINE_SMOOTH_HINT, Gl.GL_NICEST);
            Gl.glLineWidth(2.0f);
            mCamera.setView();
        }

        /**
        * Draws a frame.
        */
        private void RenderScene()
        {
            int error = Gl.glGetError();
            mFbo1.initForRendering(0);
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            error = Gl.glGetError();
            // Clear Screen And Depth Buffer
            //
            error = Gl.glGetError();
            if (mCamera != null)
            {
                mCamera.setCamera();
            }

            Gl.glPushMatrix();
            if (selectionMode)
            {
                //Gl.glPointSize(3.0f);
                //Gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

                if ((StartPoint != null) && (EndPoint != null))
                {
                    Gl.glBegin(Gl.GL_LINES);
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(StartPoint.toFloatArray());
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(EndPoint.toFloatArray());
                    Gl.glEnd();
                    //Console.WriteLine("RenderLine: " + Gl.glGetError());
                }
                else if ((StartPoint != null) && (HoverPoint != null))
                {
                    Gl.glBegin(Gl.GL_LINES);
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(StartPoint.toFloatArray());
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(HoverPoint.toFloatArray());
                    Gl.glEnd();
                    //Console.WriteLine("RenderLine: " + Gl.glGetError());
                }
            }
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            if (mLayoutLoaded)
            {
                mTreeLayout.draw(false);
            }
            error = Gl.glGetError();
            Gl.glPopMatrix();

            error = Gl.glGetError();
            mFbo1.deinitAfterRendering();
            //mFbo.pingPong(1, 0);
            mFbo2.initForRendering(0);

            // Clear Screen And Depth Buffer
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT |
            Gl.GL_DEPTH_BUFFER_BIT);

            Gl.glPushMatrix();
            if (selectionMode)
            {
                //Gl.glPointSize(3.0f);
                //Gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

                if ((StartPoint != null) && (EndPoint != null))
                {
                    Gl.glBegin(Gl.GL_LINES);
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(StartPoint.toFloatArray());
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(EndPoint.toFloatArray());
                    Gl.glEnd();
                    //Console.WriteLine("RenderLine: " + Gl.glGetError());
                }
                else if ((StartPoint != null) && (HoverPoint != null))
                {
                    Gl.glBegin(Gl.GL_LINES);
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(StartPoint.toFloatArray());
                    Gl.glColor3f(0.0f, 0.0f, 0.0f);
                    Gl.glVertex3fv(HoverPoint.toFloatArray());
                    Gl.glEnd();
                    //Console.WriteLine("RenderLine: " + Gl.glGetError());
                }
            }
            Gl.glPopMatrix();

            Gl.glPushMatrix();
            if (mLayoutLoaded)
            {
                mTreeLayout.draw(true);
            }
            Gl.glPopMatrix();
            mFbo2.deinitAfterRendering();

            GiWEngine.Base.EngineCore.Instance.mRenderStateManagerContent.disableCulling();
            GiWEngine.Base.EngineCore.Instance.mRenderStateManagerContent.disableZWrite();


            GiWEngine.Base.EngineCore.Instance.mRenderStateManagerContent.loadIdentities();

            mEffect.enableVertexProgram();
            int[] im = { mFbo1.textures[0], mFbo2.textures[0] };
            mEffect.enableFragmentProgram(im);

            Gl.glBegin(Gl.GL_QUADS);
            {
                Gl.glTexCoord2i(0, 0);
                Gl.glVertex2i(-1, -1);

                Gl.glTexCoord2i(1, 0);
                Gl.glVertex2i(1, -1);

                Gl.glTexCoord2i(1, 1);
                Gl.glVertex2i(1, 1);

                Gl.glTexCoord2i(0, 1);
                Gl.glVertex2i(-1, 1);
            }
            Gl.glEnd();

            mEffect.disableVertexProgram();
            mEffect.disableFragmentProgram();

            error = Gl.glGetError();

            GiWEngine.Base.EngineCore.Instance.mRenderStateManagerContent.enableBackfaceCulling();
            GiWEngine.Base.EngineCore.Instance.mRenderStateManagerContent.enableZWrite();
            
            Gl.glFlush();
            //location++;
            //Console.WriteLine("GlError: " + Gl.glGetError());
        }



        /// <summary>
        /// Handles the resizing of the form.
        /// </summary>
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            this.simpleOpenGlControl1.Height = this.Height - 60;
            this.simpleOpenGlControl1.Width = this.Width - 255;
            if (mCamera != null)
            {
                mCamera.mWidthContent = this.simpleOpenGlControl1.Width;
                mCamera.mHeightContent = this.simpleOpenGlControl1.Height;
                mCamera.setView();
            }
            this.Refresh();
            //RenderScene();
        }

        /**
         * Handles painting of the form.
         */
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

        }
        /**
         * Handles painting of the OpenGlControl
         */
        private void simpleOpenGlControl1_Paint(object sender, PaintEventArgs e)
        {
            mCamera.setView();
            RenderScene();
        }

        /**
         * Handles the another timer tick
         */
        private void timer1_Tick(object sender, EventArgs e)
        {
            if ((lastresult != "")/* && (!this.toolTip1.Active)*/)
            {
                //this.toolTip1.Active = true;
                //this.toolTip1.Show(lastresult, this, mCurrentX, mCurrentY, 2000000);
                string[] tmp = lastresult.Split(',');
                this.label6.Text = tmp[0];
                this.label7.Text = tmp[1];
            }
            else
            {
                this.label6.Text = "";
                this.label7.Text = "";
            }
            /*else if (result == "")
            {
                this.toolTip1.Active = false;
            }*/
            this.simpleOpenGlControl1.Refresh();
        }

        /**
         * Sets the beta value for flattening the splines
         */
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            float beta = ((float)trackBar1.Value) / 25.0f;
            this.label3.Text = "boundling strength: " + beta;
            if (mTreeLayout != null)
            {
                mTreeLayout.updateBoundlingStrength(beta, checkBox1.Checked, checkBox2.Checked);
            }
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            this.textBox1.Text = this.openFileDialog1.FileName;
            this.button2.Enabled = true;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.openFileDialog1.ShowDialog();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (this.openFileDialog1.CheckFileExists)
            {
                //TODO: load file
                mTree = Tree.Tree.loadFromFile(this.openFileDialog1.FileName);
                mTree.enumerateLevels();
            }
            this.button3.Enabled = true;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            radialTree = radioButton1.Checked;
        }

        /**
         * Constructs the selected Layout
         */
        private void button3_Click(object sender, EventArgs e)
        {
            if (radialTree)
            {
                mTreeLayout = new RadialTreeLayout(ref mTree);
                mLayoutLoaded = mTreeLayout.computeLayout();
                mTreeLayout.generateTreeList();
                mTreeLayout.generateAdjacencyControlPoints();
                float beta = ((float)trackBar1.Value) / 25.0f;
                mTreeLayout.updateBoundlingStrength(beta, checkBox1.Checked, checkBox2.Checked);
            }
            else
            {
                mTreeLayout = new TreeMapLayout(ref mTree, this.simpleOpenGlControl1.Width, this.simpleOpenGlControl1.Height);
                mLayoutLoaded = mTreeLayout.computeLayout();
                mTreeLayout.generateTreeList();
                mTreeLayout.generateAdjacencyControlPoints();
                float beta = ((float)trackBar1.Value) / 25.0f;
                mTreeLayout.updateBoundlingStrength(beta, checkBox1.Checked, checkBox2.Checked);
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (mTreeLayout != null)
            {
                float beta = ((float)trackBar1.Value) / 25.0f;
                mTreeLayout.updateBoundlingStrength(beta, checkBox1.Checked, checkBox2.Checked);
            }
        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            if (mTreeLayout != null)
            {
                float beta = ((float)trackBar1.Value) / 25.0f;
                mTreeLayout.updateBoundlingStrength(beta, checkBox1.Checked, checkBox2.Checked);
            }
        }

        private void InfoVis_Paint(object sender, PaintEventArgs e)
        {
        }

        /**
         * Starts Parsing the given Directory for Code files
         */
        private void button5_Click(object sender, EventArgs e)
        {
            this.saveFileDialog1.ShowDialog();
            StreamWriter output = new StreamWriter(this.saveFileDialog1.FileName);
            DirectoryInfo dir = new DirectoryInfo(this.saveFileDialog1.FileName.Substring(0, this.saveFileDialog1.FileName.LastIndexOf('\\')));
            mNodes = new Dictionary<string, int>();

            int id = 0;
            string rootNode = this.saveFileDialog1.FileName.Substring(0, this.saveFileDialog1.FileName.LastIndexOf('\\'));
            output.WriteLine(id + " " + rootNode);
            mNodes.Add(rootNode, id);
            id++;
            //Parse C++
            //write initial Nodes
            FileInfo[] files = dir.GetFiles();
            foreach (FileInfo fi in files)
            {
                if (fi.FullName == this.saveFileDialog1.FileName)
                {
                    continue;
                }
                if (radioButton3.Checked)
                {
                    if ((fi.Extension == ".hpp") || (fi.Extension == ".c") || (fi.Extension == ".cpp") || (fi.Extension == ".cxx") || (fi.Extension == ".h") || (fi.Extension == ".cc"))
                    {
                        output.WriteLine(id + " " + fi.Name);
                        mNodes.Add(fi.Name, id);
                        id++;
                    }
                }
                else if (radioButton4.Checked)
                {
                    if ((fi.Extension == ".c") || (fi.Extension == ".m") || (fi.Extension == ".h"))
                    {
                        output.WriteLine(id + " " + fi.Name);
                        mNodes.Add(fi.Name, id);
                        id++;
                    }
                }
            }
            DirectoryInfo[] dirs = dir.GetDirectories();
            foreach (DirectoryInfo di in dirs)
            {
                if (!((di.Name.Equals(".svn")) || (di.Name.Equals("CVS"))))
                {
                    FileInfo[] tmpDirs;
                    if (radioButton3.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.cpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cxx", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cc", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.hpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    else if (radioButton4.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.m", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    id = parseDir(rootNode, id, di, output);
                }
            }
            //write hierarchy
            output.WriteLine("inheritance");
            foreach (FileInfo fi in files)
            {
                int NodeId;
                if (mNodes.TryGetValue(fi.Name, out NodeId))
                {
                    output.WriteLine("0 " + NodeId);
                }
            }
            foreach (DirectoryInfo di in dirs)
            {
                if (!((di.Name.Equals(".svn")) || (di.Name.Equals("CVS"))))
                {
                    FileInfo[] tmpDirs;
                    if (radioButton3.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.cpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cxx", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cc", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.hpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    else if (radioButton4.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.m", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    parseHierarchy(0, rootNode, di, output);
                }
            }
            //write adjacency
            output.WriteLine("adjacency");
            foreach (FileInfo fi in files)
            {
                if (fi.FullName == this.saveFileDialog1.FileName)
                {
                    continue;
                }
                //look for includes
                int NodeId;
                if (mNodes.TryGetValue(fi.Name, out NodeId))
                {
                    try
                    {
                        int nodeId;
                        if (mNodes.TryGetValue(fi.Name, out nodeId))
                        {
                            StreamReader sr = new StreamReader(fi.OpenRead());

                            while (!sr.EndOfStream)
                            {
                                string line = sr.ReadLine();
                                string filename = "";
                                if (line.StartsWith("#include \"") || line.StartsWith("#include\""))
                                {
                                    filename = line.Substring(line.IndexOf('"') + 1, line.LastIndexOf('"') - line.IndexOf('"') - 1);

                                }
                                else if (line.StartsWith("#include <") || line.StartsWith("#include<"))
                                {
                                    filename = line.Substring(line.IndexOf('<') + 1, line.LastIndexOf('>') - line.IndexOf('<') - 1);
                                }
                                if (filename != "")
                                {
                                    if (filename.Contains("\\") || filename.Contains("/"))
                                    {
                                        filename = filename.Replace('/', '\\');
                                        if (!filename.StartsWith("\\"))
                                        {
                                            filename = "\\" + filename;
                                        }
                                    }
                                    int fileNodeId;
                                    if (mNodes.TryGetValue(filename, out fileNodeId))
                                    {
                                        if (!(nodeId == fileNodeId))
                                        {
                                            output.WriteLine(nodeId + " " + fileNodeId);
                                        }
                                    }
                                    else
                                    {
                                        //try out different things with included files
                                        string[] filenameparts = filename.Split('\\');
                                        FileInfo[] mfiles = dir.GetFiles(filenameparts[filenameparts.Length - 1], SearchOption.AllDirectories);
                                        if (mfiles.Length != 0)
                                        {
                                            foreach (FileInfo mfi in mfiles)
                                            {
                                                string partFileName = mfi.FullName.Remove(0, rootNode.Length);
                                                int mNodeId;
                                                if (mNodes.TryGetValue(partFileName, out mNodeId))
                                                {
                                                    if (!(nodeId == mNodeId))
                                                    {
                                                        output.WriteLine(nodeId + " " + mNodeId);
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            sr.Close();
                        }
                    }
                    catch (Exception)
                    {
                        GiWEngine.Base.EngineCore.Instance.logger.printLine("File " + fi.FullName + " cannot be opened for Reading");
                    }
                }

            }
            foreach (DirectoryInfo di in dirs)
            {
                if (!((di.Name.Equals(".svn")) || (di.Name.Equals("CVS"))))
                {
                    FileInfo[] tmpDirs;
                    if (radioButton3.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.cpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cxx", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cc", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.hpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    else if (radioButton4.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.m", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    parseAdjacency(dir, 0, rootNode, di, output);
                }
            }
            output.WriteLine("");
            output.Close();
        }

        /**
         * Parses a Subdirectory for files and additional Subdirs
         * @param rootNode Current Node = Current Dir
         * @param id id of Current Node
         * @param dir Directory Infor Object for current dir
         * @param output Streamwriter for writing the output
         */
        private int parseDir(string rootNode, int id, DirectoryInfo dir, StreamWriter output)
        {
            string tmp = dir.FullName.Remove(0, rootNode.Length);
            output.WriteLine(id + " " + tmp);
            mNodes.Add(tmp, id);
            id++;
            FileInfo[] files = dir.GetFiles();
            foreach (FileInfo fi in files)
            {
                if (radioButton3.Checked)
                {
                    if ((fi.Extension == ".hpp") || (fi.Extension == ".c") || (fi.Extension == ".cpp") || (fi.Extension == ".cxx") || (fi.Extension == ".h") || (fi.Extension == ".cc"))
                    {
                        output.WriteLine(id + " " + tmp + "\\" + fi.Name);
                        mNodes.Add(tmp + "\\" + fi.Name, id);
                        id++;
                    }
                }
                else if (radioButton4.Checked)
                {
                    //Parse Obj C
                    if ((fi.Extension == ".c") || (fi.Extension == ".m") || (fi.Extension == ".h"))
                    {
                        output.WriteLine(id + " " + tmp + "\\" + fi.Name);
                        mNodes.Add(tmp + "\\" + fi.Name, id);
                        id++;
                    }
                }
            }
            DirectoryInfo[] dirs = dir.GetDirectories();
            foreach (DirectoryInfo di in dirs)
            {
                if (!((di.Name.Equals(".svn")) || (di.Name.Equals("CVS"))))
                {
                    FileInfo[] tmpDirs;
                    if (radioButton3.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.cpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cxx", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cc", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.hpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    else if (radioButton4.Checked)
                    {
                        if (!(((tmpDirs = di.GetFiles("*.m", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                            ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                        {
                            continue;
                        }
                    }
                    id = parseDir(rootNode, id, di, output);
                }

            }
            return id;
        }

        /**
         * Parses the hierarchy given via the Directory Structure
         * @param rootNode Current Node = Current Dir
         * @param rootNodeId id of Current Node
         * @param dir Directory Infor Object for current dir
         * @param output Streamwriter for writing the output
         */
        private void parseHierarchy(int rootNodeId, string rootNode, DirectoryInfo dir, StreamWriter output)
        {
            string tmp = dir.FullName.Remove(0, rootNode.Length);
            int id;
            if (mNodes.TryGetValue(tmp, out id))
            {
                output.WriteLine(rootNodeId + " " + id);
                FileInfo[] files = dir.GetFiles();
                foreach (FileInfo fi in files)
                {
                    int NodeId;
                    if (mNodes.TryGetValue(tmp + "\\" + fi.Name, out NodeId))
                    {
                        output.WriteLine(id + " " + NodeId);
                    }
                }
                DirectoryInfo[] dirs = dir.GetDirectories();
                foreach (DirectoryInfo di in dirs)
                {
                    if (!((di.Name.Equals(".svn")) || (di.Name.Equals("CVS"))))
                    {
                        FileInfo[] tmpDirs;
                        if (radioButton3.Checked)
                        {
                            if (!(((tmpDirs = di.GetFiles("*.cpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cxx", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cc", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.hpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                            {
                                continue;
                            }
                        }
                        else if (radioButton4.Checked)
                        {
                            if (!(((tmpDirs = di.GetFiles("*.m", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                            {
                                continue;
                            }
                        }
                        parseHierarchy(id, rootNode, di, output);
                    }

                }
            }
        }

        /**
         * Parses the Adjacency for all Files and Subdirs found
         * @param rootDir Directory info for the Root Dir
         * @param rootNode Current Node = Current Dir
         * @param id id of Current Node
         * @param dir Directory Infor Object for current dir
         * @param output Streamwriter for writing the output
         */
        private void parseAdjacency(DirectoryInfo rootDir, int rootNodeId, string rootNode, DirectoryInfo dir, StreamWriter output)
        {
            string tmp = dir.FullName.Remove(0, rootNode.Length);
            int id;
            if (mNodes.TryGetValue(tmp, out id))
            {
                FileInfo[] files = dir.GetFiles();
                foreach (FileInfo fi in files)
                {
                    //look for includes
                    try
                    {
                        int nodeId;
                        if (mNodes.TryGetValue(tmp + "\\" + fi.Name, out nodeId))
                        {
                            StreamReader sr = new StreamReader(fi.OpenRead());

                            while (!sr.EndOfStream)
                            {
                                string line = sr.ReadLine();
                                string filename = "";
                                if (line.StartsWith("#include \"") || line.StartsWith("#include\""))
                                {
                                    filename = line.Substring(line.IndexOf('"') + 1, line.LastIndexOf('"') - line.IndexOf('"') - 1);

                                }
                                else if (line.StartsWith("#include <") || line.StartsWith("#include<"))
                                {
                                    filename = line.Substring(line.IndexOf('<') + 1, line.LastIndexOf('>') - line.IndexOf('<') - 1);
                                }
                                if (filename != "")
                                {
                                    if (filename.Contains("\\") || filename.Contains("/"))
                                    {
                                        filename = filename.Replace('/', '\\');
                                        if (!filename.StartsWith("\\"))
                                        {
                                            filename = "\\" + filename;
                                        }
                                    }
                                    int fileNodeId;
                                    if (mNodes.TryGetValue(tmp + "\\" + filename, out fileNodeId))
                                    {
                                        if (!(nodeId == fileNodeId))
                                        {
                                            output.WriteLine(nodeId + " " + fileNodeId);
                                        }
                                    }
                                    else
                                    {
                                        //try out different things with included files
                                        string[] filenameparts = filename.Split('\\');
                                        FileInfo[] mfiles = rootDir.GetFiles(filenameparts[filenameparts.Length - 1], SearchOption.AllDirectories);
                                        if (mfiles.Length != 0)
                                        {
                                            foreach (FileInfo mfi in mfiles)
                                            {
                                                string partFileName = mfi.FullName.Remove(0, rootNode.Length);
                                                int NodeId;
                                                if (mNodes.TryGetValue(partFileName, out NodeId))
                                                {
                                                    if (!(nodeId == NodeId))
                                                    {
                                                        output.WriteLine(nodeId + " " + NodeId);
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            sr.Close();
                        }
                    }
                    catch (Exception)
                    {
                        GiWEngine.Base.EngineCore.Instance.logger.printLine("File " + fi.FullName + " cannot be opened for Reading");
                    }

                }
                DirectoryInfo[] dirs = dir.GetDirectories();
                foreach (DirectoryInfo di in dirs)
                {
                    if (!((di.Name.Equals(".svn")) || (di.Name.Equals("CVS"))))
                    {
                        FileInfo[] tmpDirs;
                        if (radioButton3.Checked)
                        {
                            if (!(((tmpDirs = di.GetFiles("*.cpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cxx", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.cc", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.hpp", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                            {
                                continue;
                            }
                        }
                        else if (radioButton4.Checked)
                        {
                            if (!(((tmpDirs = di.GetFiles("*.m", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.c", SearchOption.AllDirectories)).Length != 0) ||
                                ((tmpDirs = di.GetFiles("*.h", SearchOption.AllDirectories)).Length != 0)))
                            {
                                continue;
                            }
                        }
                        parseAdjacency(rootDir, id, rootNode, di, output);
                    }

                }
            }
        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e)
        {
            selectionMode = !selectionMode;
            
            StartPoint = null;
            EndPoint = null;
            HoverPoint = null;
            mTreeLayout.selectWithLine(null, null);
            mTreeLayout.setSelectedNode(null);
            
        }
    }
}