using System;
using System.Collections.Generic;
using System.Text;
using Tao.OpenGl;

namespace InfoVis.Tree
{
    /**
     * Class representing a TreeMaplayout
     */
    public class TreeMapLayout : TreeLayout
    {
        #region static variables
        public static float borderwidth = 0.5f;
        #endregion

        #region Member Variables
        private float mWidthHalf; /**< half width of window */
        private float mHeightHalf;/**< half height of window */ 
        #endregion

        #region Constructor
        /**
         * CTR of a Treemap Layout
         * @param m_Tree Tree
         * @param windowWidth width of current window
         * @param windowHeight height of current window
         */
        public TreeMapLayout(ref Tree m_Tree, int windowWidth, int windowHeight)
        {
            mTree = m_Tree;
            mWidthHalf = (float)windowWidth / 20.0f * (/*10.0f /*/ mTree.mNodesPerLevelContent.Count - 1);
            mHeightHalf = (float)windowHeight / 20.0f * (/*10.0f /*/ mTree.mNodesPerLevelContent.Count - 1);
            mLinesSelected = false;
        }
        #endregion

        #region Methods
        /**
         * Computes the SubnodeLayout for a given Subnode
         * @param subNode Current active Subnode
         * @param rect Rectangle of current Node
         * @param centerpos Position of center of Rectangle
         * @param level number of current treelevel
         */
        private void computeSubNodeLayout(TreeNode subNode, List<GiWEngine.Math.Vector3> rect, GiWEngine.Math.Vector3 centerpos, int level)
        {
            subNode.mPositionContent = centerpos;
            subNode.mSphereSizeContent = 0.1f;
            subNode.setRectangeCoordinates(rect);
            if (!subNode.mIsLeafContent)
            {
                //creating initial subnoderectangle
                int numSubnodes = subNode.mSubNodesContent.Count;
                float width = (float)rect[2].x - rect[0].x;
                float height = (float)rect[2].y - rect[0].y;
                
                float eachwidth;
                float eachheight;

                if((level %2) == 0)
                {
                    float nettowidth = width - (2.0f * borderwidth) - (numSubnodes * borderwidth);
                    eachwidth = nettowidth / numSubnodes;
                    eachheight = height - (2.0f * borderwidth);
                }
                else if((level %2) == 1)
                {
                    float nettoheight = height - (2.0f * borderwidth) - (numSubnodes * borderwidth);
                    eachwidth = width - (2.0f * borderwidth);
                    eachheight = nettoheight / numSubnodes;
                }
                else
                {
                    return;
                }

                GiWEngine.Math.Vector3 startMin = new GiWEngine.Math.Vector3(rect[0].x + borderwidth, rect[0].y + borderwidth, 0.0f);
                GiWEngine.Math.Vector3 startMax = new GiWEngine.Math.Vector3(rect[0].x + borderwidth + eachwidth, rect[0].y + borderwidth + eachheight, 0.0f);


                foreach (KeyValuePair<int, TreeNode> kvp in subNode.mSubNodesContent)
                {
                    List<GiWEngine.Math.Vector3> rectangle = new List<GiWEngine.Math.Vector3>();
                    rectangle.Add(startMin);
                    rectangle.Add(new GiWEngine.Math.Vector3(startMax.x, startMin.y, 0.0f));
                    rectangle.Add(startMax);
                    rectangle.Add(new GiWEngine.Math.Vector3(startMin.x, startMax.y, 0.0f));
                    GiWEngine.Math.Vector3 center = startMin + ((startMax - startMin)/2.0f);
                    
                    computeSubNodeLayout(kvp.Value, rectangle, center, level+1);
                    if ((level % 2) == 0)
                    {
                        startMin += new GiWEngine.Math.Vector3(eachwidth + borderwidth, 0.0f, 0.0f);
                        startMax += new GiWEngine.Math.Vector3(eachwidth + borderwidth, 0.0f, 0.0f);
                    }
                    else if ((level % 2) == 1)
                    {
                        startMin += new GiWEngine.Math.Vector3(0.0f, eachheight + borderwidth, 0.0f);
                        startMax += new GiWEngine.Math.Vector3(0.0f, eachheight + borderwidth, 0.0f);
                    }
                }
            }
        }

        
        #endregion

        #region overridden Methods
        /**
         * Method for Computing the current Layout
         */
        public override bool computeLayout()
        {
            TreeNode root = mTree.mRootNodeContent;
            root.mPositionContent = new GiWEngine.Math.Vector3(0.0f, 0.0f, 0.0f);

            List<GiWEngine.Math.Vector3> rect = new List<GiWEngine.Math.Vector3>();
            rect.Add(new GiWEngine.Math.Vector3(-(mWidthHalf-5), -(mHeightHalf-5), 0.0f));
            rect.Add(new GiWEngine.Math.Vector3((mWidthHalf - 5), -(mHeightHalf - 5), 0.0f));
            rect.Add(new GiWEngine.Math.Vector3((mWidthHalf - 5), (mHeightHalf - 5), 0.0f));
            rect.Add(new GiWEngine.Math.Vector3(-(mWidthHalf - 5), (mHeightHalf - 5), 0.0f));
            root.setRectangeCoordinates(rect);
            root.mSphereSizeContent = 0.3f;

            //creating initial subnoderectangle
            int numSubnodes = root.mSubNodesContent.Count;
            float width = (float)(mWidthHalf - 5) * 2.0f;
            float height = (float)(mHeightHalf - 5) * 2.0f;
            float nettowidth = width - (2.0f * borderwidth) - (numSubnodes * borderwidth);
            float eachwidth = nettowidth / numSubnodes;
            float eachheight = height - (2.0f * borderwidth);

            GiWEngine.Math.Vector3 startMin = new GiWEngine.Math.Vector3(-(mWidthHalf - 5) + borderwidth, -(mHeightHalf - 5) + borderwidth, 0.0f);
            GiWEngine.Math.Vector3 startMax = new GiWEngine.Math.Vector3(-(mWidthHalf - 5) + borderwidth + eachwidth, -(mHeightHalf - 5) + borderwidth + eachheight, 0.0f);


            foreach (KeyValuePair<int, TreeNode> kvp in root.mSubNodesContent)
            {
                List<GiWEngine.Math.Vector3> rectangle = new List<GiWEngine.Math.Vector3>();
                rectangle.Add(startMin);
                rectangle.Add(new GiWEngine.Math.Vector3(startMax.x, startMin.y, 0.0f));
                rectangle.Add(startMax);
                rectangle.Add(new GiWEngine.Math.Vector3(startMin.x, startMax.y, 0.0f));
                GiWEngine.Math.Vector3 center = startMin + ((startMax - startMin)/2.0f);
                
                computeSubNodeLayout(kvp.Value, rectangle, center, 1);

                startMin += new GiWEngine.Math.Vector3(eachwidth + borderwidth, 0.0f, 0.0f);
                startMax += new GiWEngine.Math.Vector3(eachwidth + borderwidth, 0.0f, 0.0f);
            }
            return true;
        }

        /**
         * Method for generating the Displaylists for Tree
         */
        public override void generateTreeList()
        {
            mTreeList = Gl.glGenLists(1);
            Gl.glNewList(mTreeList, Gl.GL_COMPILE);
            Gl.glColor4f(0.9f, 0.9f, 0.9f, 0.9f);
            mTree.mRootNodeContent.genDisplayList();
            Gl.glEndList();

            mTreeNodeList = Gl.glGenLists(1);
            Gl.glNewList(mTreeNodeList, Gl.GL_COMPILE);

            Gl.glShadeModel(Gl.GL_SMOOTH);

            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);
            float[] diffuseLight = { 0.8f, 0.8f, 0.8f, 1.0f };
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diffuseLight);
            float[] specular = { 1.0f, 1.0f, 1.0f, 1.0f };
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_SPECULAR, specular);
            float[] ambient = { 1.0f, 1.0f, 1.0f };
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, ambient);
            float[] lightpos = { 0.0f, 5.0f, -20.0f };
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, lightpos);

            mTree.mRootNodeContent.draw();

            Gl.glDisable(Gl.GL_LIGHTING);
            Gl.glDisable(Gl.GL_LIGHT0);
            Gl.glEndList();
        }
        #endregion
    }
}
